package net.wanji.utc.cache;

import lombok.RequiredArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import net.wanji.databus.dao.entity.CrossLightsPO;
import net.wanji.databus.dao.entity.CrossPhaseLightsPO;
import net.wanji.databus.dao.entity.CrossPhasePO;
import net.wanji.databus.dao.entity.CrossSchemePO;
import net.wanji.databus.dao.mapper.*;
import net.wanji.databus.po.CrossInfoPO;
import net.wanji.utc.po.hk.CrossPhaseDirTurnPojo;
import org.apache.commons.lang3.StringUtils;
import org.springframework.boot.CommandLineRunner;
import org.springframework.stereotype.Component;
import org.springframework.util.CollectionUtils;

import java.util.*;
import java.util.stream.Collectors;

/**
 * @author duanruiming
 * @date 2023/09/11 17:42
 */
@Slf4j
@Component
@SuppressWarnings("unchecked")
@RequiredArgsConstructor
public class CrossPhaseDirTurnCache implements CommandLineRunner {

    private final CrossInfoMapper crossInfoMapper;
    private final CrossPhaseMapper crossPhaseMapper;
    private final CrossSchemeMapper crossSchemeMapper;
    private final CrossPhaseLightsMapper crossPhaseLightsMapper;
    private final CrossLightsMapper crossLightsMapper;

    private static final Map<String, List<CrossPhaseDirTurnPojo>> crossInfoMap = new HashMap<>();

    public static List<CrossPhaseDirTurnPojo> getCrossPhaseDirTurnCache(String crossId) {
        if (!crossInfoMap.isEmpty()) {
            return crossInfoMap.get(crossId);
        }
        return null;
    }

    @Override
    public void run(String... args) throws Exception {
        init();
    }

    public void init() {
        List<CrossInfoPO> crossInfoPOList = crossInfoMapper.selectAll();
        if (!CollectionUtils.isEmpty(crossInfoPOList)) {
            for (CrossInfoPO crossInfoPO : crossInfoPOList) {
                String crossId = crossInfoPO.getId();
                List<CrossSchemePO> crossSchemePOS = crossSchemeMapper.selectByCrossId(crossId);
                Map<Integer, String> schemeIdNoMap = crossSchemePOS.stream().collect(Collectors.toMap(CrossSchemePO::getId, CrossSchemePO::getSchemeNo));
                List<CrossPhasePO> crossPhasePOS = crossPhaseMapper.selectByCrossId(crossId);
                if (!CollectionUtils.isEmpty(crossPhasePOS)) {
                    List<CrossPhaseDirTurnPojo> results = new ArrayList<>();
                    for (CrossPhasePO crossPhasePO : crossPhasePOS) {
                        CrossPhaseDirTurnPojo crossPhaseDirTurnPojo = new CrossPhaseDirTurnPojo();
                        String phaseNo = crossPhasePO.getPhaseNo();
                        crossPhaseDirTurnPojo.setCrossId(crossId);
                        crossPhaseDirTurnPojo.setPhaseNo(Integer.valueOf(phaseNo));
                        Integer planId = crossPhasePO.getPlanId();
                        String schemeNo = schemeIdNoMap.get(planId);
                        if (StringUtils.isNotBlank(schemeNo)) {
                            crossPhaseDirTurnPojo.setSchemeNo(Integer.valueOf(schemeNo));
                        } else {
                            crossPhaseDirTurnPojo.setSchemeNo(-1);
                        }

                        Integer phaseId = crossPhasePO.getId();
                        List<CrossPhaseLightsPO> crossPhaseLightsPOList = crossPhaseLightsMapper.selectByCrossId(crossId);
                        List<Integer> phaseLightsList = crossPhaseLightsPOList.stream()
                                .filter(po -> Objects.equals(po.getPhaseId(), phaseId))
                                .map(CrossPhaseLightsPO::getLightsId).collect(Collectors.toList());

                        List<CrossLightsPO> crossLightsPOS = crossLightsMapper.selectByCrossId(crossId);
                        Map<Integer, List<CrossLightsPO>> phaseDirMap = crossLightsPOS.stream().filter(po -> phaseLightsList.contains(po.getId())).collect(Collectors.groupingBy(CrossLightsPO::getDir));
                        Map<Integer, List<CrossLightsPO>> dirTurnMap = new HashMap<>();
                        for (Map.Entry<Integer, List<CrossLightsPO>> entry : phaseDirMap.entrySet()) {
                            Integer dir = entry.getKey();
//                            List<Integer> turnList = entry.getValue().stream().map(CrossLightsPO::getTurn).sorted().collect(Collectors.toList());
//                            dirTurnMap.put(dir, turnList);
                            dirTurnMap.put(dir, entry.getValue());
                            crossPhaseDirTurnPojo.setDirTurnMap(dirTurnMap);
                        }
                        results.add(crossPhaseDirTurnPojo);
                    }
                    crossInfoMap.put(crossId, results);
                }
            }
        }
    }

}
